#include "PrefWindow.hpp"
#include "core.hpp"
#include <array>
#include <fstream>
#include <iostream>
#include <gtkmm/stringlist.h>
#include <assert.h>
#include <nlohmann/json.hpp>
#include "app.hpp"
#include <YoutubeApi/youtubeapi.hpp>

using json = nlohmann::json;

namespace components {
    PrefWindow::PrefWindow(
        BaseObjectType* cobject,
        const Glib::RefPtr<Gtk::Builder>& refBuilder)
        : Gtk::Window(cobject)
        , m_builder(refBuilder)
    {
        // Заполнение выпадающего списка качества видео
        std::vector<Glib::ustring> quality_strings =
        {
            "240p",
            "360p",
            "480p",
            "720p",
            "1080p"
        };
        
        m_quality_dropdown = m_builder->get_widget<Gtk::DropDown>("quality_entry");
        m_cancel_btn = m_builder->get_widget<Gtk::Button>("cancel_button");
        m_save_btn = m_builder->get_widget<Gtk::Button>("saveclose_button");
        m_apikey_entry = m_builder->get_widget<Gtk::Entry>("apikey_entry");

        assert(m_quality_dropdown != nullptr);
        assert(m_cancel_btn != nullptr);
        assert(m_save_btn != nullptr);
        assert(m_apikey_entry != nullptr);

        auto string_list = Gtk::StringList::create(quality_strings);
        m_quality_dropdown->set_model(string_list);

        m_save_btn->signal_clicked().connect(sigc::mem_fun(
            *this, &PrefWindow::onSaveClicked));
        m_cancel_btn->signal_clicked().connect(sigc::mem_fun(
            *this, &PrefWindow::onCancelClicked));

        m_apikey_entry->set_text(core::Settings::getApiKey());

        int selected_quality_index;
        switch(core::Settings::getQuality())
        {
            case 240:
                selected_quality_index = 0;
                break;
            case 360:
                selected_quality_index = 1;
                break;
            case 480:
                selected_quality_index = 2;
                break;
            case 720:
                selected_quality_index = 3;
                break;
            case 1080:
                selected_quality_index = 4;
                break;
            default:
                selected_quality_index = 0;
                break;
        }
        m_quality_dropdown->set_selected(selected_quality_index);
    }

    PrefWindow* PrefWindow::create(
        Gtk::Window& parent
    )
    {
        auto builder = Gtk::Builder::create_from_resource("/src/YTMPV/res/PrefsWindow.ui");
        auto win = Gtk::Builder::get_widget_derived<PrefWindow>(builder, "prefs_dialog");
        assert(win != nullptr);

        win->set_transient_for(parent);
        return win;
    }

    void PrefWindow::onCancelClicked()
    {
        this->close();
    }

    void PrefWindow::onSaveClicked()
    {
        this->saveConfig();
        this->close();
    }

    void PrefWindow::saveConfig()
    {
        // -- Получение данных --
        // Ключ API
        Glib::ustring api_key = m_apikey_entry->get_text();

        // Качество
        int quality;
        int sel_index = m_quality_dropdown->get_selected();
        switch(sel_index)
        {
            case 0:
                quality = 240;
                break;
            case 1:
                quality = 360;
                break;
            case 2:
                quality = 480;
                break;
            case 3:
                quality = 720;
                break;
            case 4:
                quality = 1080;
                break;
            default:
                quality = 240;
                break;
        }
        
        // Сериализация
        json j_object = { {"apikey", api_key}, {"quality", quality} };
        std::string dump = j_object.dump(1, '\t');

        // Запись в файл
        std::ofstream config(core::Settings::getConfigPath());
        config << dump;
        config.close();

        // Пере-инициализация библиотек
        ytapi::youtubeInit(api_key);
        core::Settings::setApiKey(api_key);
        core::Settings::setQuality(quality);
    }
}
